# frozen_string_literal: true

# WARNING ABOUT GENERATED CODE
#
# This file is generated. See the contributing guide for more information:
# https://github.com/aws/aws-sdk-ruby/blob/master/CONTRIBUTING.md
#
# WARNING ABOUT GENERATED CODE

module Aws::CloudFormation

  class StackResource

    extend Aws::Deprecations

    # @overload def initialize(stack_name, logical_id, options = {})
    #   @param [String] stack_name
    #   @param [String] logical_id
    #   @option options [Client] :client
    # @overload def initialize(options = {})
    #   @option options [required, String] :stack_name
    #   @option options [required, String] :logical_id
    #   @option options [Client] :client
    def initialize(*args)
      options = Hash === args.last ? args.pop.dup : {}
      @stack_name = extract_stack_name(args, options)
      @logical_id = extract_logical_id(args, options)
      @data = options.delete(:data)
      @client = options.delete(:client) || Client.new(options)
      @waiter_block_warned = false
    end

    # @!group Read-Only Attributes

    # @return [String]
    def stack_name
      @stack_name
    end

    # @return [String]
    def logical_id
      @logical_id
    end
    alias :logical_resource_id :logical_id

    # Unique identifier of the stack.
    # @return [String]
    def stack_id
      data[:stack_id]
    end

    # The name or unique identifier that corresponds to a physical instance
    # ID of a resource supported by AWS CloudFormation.
    # @return [String]
    def physical_resource_id
      data[:physical_resource_id]
    end

    # Type of resource. ((For more information, go to [ AWS Resource Types
    # Reference][1] in the AWS CloudFormation User Guide.)
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-template-resource-type-ref.html
    # @return [String]
    def resource_type
      data[:resource_type]
    end

    # Time the status was updated.
    # @return [Time]
    def last_updated_timestamp
      data[:last_updated_timestamp]
    end

    # Current status of the resource.
    # @return [String]
    def resource_status
      data[:resource_status]
    end

    # Success/failure message associated with the resource.
    # @return [String]
    def resource_status_reason
      data[:resource_status_reason]
    end

    # User defined description associated with the resource.
    # @return [String]
    def description
      data[:description]
    end

    # The content of the `Metadata` attribute declared for the resource. For
    # more information, see [Metadata Attribute][1] in the AWS
    # CloudFormation User Guide.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-attribute-metadata.html
    # @return [String]
    def metadata
      data[:metadata]
    end

    # Information about whether the resource's actual configuration
    # differs, or has *drifted*, from its expected configuration, as defined
    # in the stack template and any values specified as template parameters.
    # For more information, see [Detecting Unregulated Configuration Changes
    # to Stacks and Resources][1].
    #
    #
    #
    # [1]: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-stack-drift.html
    # @return [Types::StackResourceDriftInformation]
    def drift_information
      data[:drift_information]
    end

    # @!endgroup

    # @return [Client]
    def client
      @client
    end

    # Loads, or reloads {#data} for the current {StackResource}.
    # Returns `self` making it possible to chain methods.
    #
    #     stack_resource.reload.data
    #
    # @return [self]
    def load
      resp = @client.describe_stack_resource(
        logical_resource_id: @logical_id,
        stack_name: @stack_name
      )
      @data = resp.stack_resource_detail
      self
    end
    alias :reload :load

    # @return [Types::StackResourceDetail]
    #   Returns the data for this {StackResource}. Calls
    #   {Client#describe_stack_resource} if {#data_loaded?} is `false`.
    def data
      load unless @data
      @data
    end

    # @return [Boolean]
    #   Returns `true` if this resource is loaded.  Accessing attributes or
    #   {#data} on an unloaded resource will trigger a call to {#load}.
    def data_loaded?
      !!@data
    end

    # @deprecated Use [Aws::CloudFormation::Client] #wait_until instead
    #
    # Waiter polls an API operation until a resource enters a desired
    # state.
    #
    # @note The waiting operation is performed on a copy. The original resource
    #   remains unchanged.
    #
    # ## Basic Usage
    #
    # Waiter will polls until it is successful, it fails by
    # entering a terminal state, or until a maximum number of attempts
    # are made.
    #
    #     # polls in a loop until condition is true
    #     resource.wait_until(options) {|resource| condition}
    #
    # ## Example
    #
    #     instance.wait_until(max_attempts:10, delay:5) do |instance|
    #       instance.state.name == 'running'
    #     end
    #
    # ## Configuration
    #
    # You can configure the maximum number of polling attempts, and the
    # delay (in seconds) between each polling attempt. The waiting condition is
    # set by passing a block to {#wait_until}:
    #
    #     # poll for ~25 seconds
    #     resource.wait_until(max_attempts:5,delay:5) {|resource|...}
    #
    # ## Callbacks
    #
    # You can be notified before each polling attempt and before each
    # delay. If you throw `:success` or `:failure` from these callbacks,
    # it will terminate the waiter.
    #
    #     started_at = Time.now
    #     # poll for 1 hour, instead of a number of attempts
    #     proc = Proc.new do |attempts, response|
    #       throw :failure if Time.now - started_at > 3600
    #     end
    #
    #       # disable max attempts
    #     instance.wait_until(before_wait:proc, max_attempts:nil) {...}
    #
    # ## Handling Errors
    #
    # When a waiter is successful, it returns the Resource. When a waiter
    # fails, it raises an error.
    #
    #     begin
    #       resource.wait_until(...)
    #     rescue Aws::Waiters::Errors::WaiterFailed
    #       # resource did not enter the desired state in time
    #     end
    #
    # @yieldparam [Resource] resource to be used in the waiting condition.
    #
    # @raise [Aws::Waiters::Errors::FailureStateError] Raised when the waiter
    #   terminates because the waiter has entered a state that it will not
    #   transition out of, preventing success.
    #
    #   yet successful.
    #
    # @raise [Aws::Waiters::Errors::UnexpectedError] Raised when an error is
    #   encountered while polling for a resource that is not expected.
    #
    # @raise [NotImplementedError] Raised when the resource does not
    #
    # @option options [Integer] :max_attempts (10) Maximum number of
    # attempts
    # @option options [Integer] :delay (10) Delay between each
    # attempt in seconds
    # @option options [Proc] :before_attempt (nil) Callback
    # invoked before each attempt
    # @option options [Proc] :before_wait (nil) Callback
    # invoked before each wait
    # @return [Resource] if the waiter was successful
    def wait_until(options = {}, &block)
      self_copy = self.dup
      attempts = 0
      options[:max_attempts] = 10 unless options.key?(:max_attempts)
      options[:delay] ||= 10
      options[:poller] = Proc.new do
        attempts += 1
        if block.call(self_copy)
          [:success, self_copy]
        else
          self_copy.reload unless attempts == options[:max_attempts]
          :retry
        end
      end
      Aws::Waiters::Waiter.new(options).wait({})
    end

    # @!group Associations

    # @return [Stack]
    def stack
      Stack.new(
        name: @stack_name,
        client: @client
      )
    end

    # @deprecated
    # @api private
    def identifiers
      {
        stack_name: @stack_name,
        logical_id: @logical_id
      }
    end
    deprecated(:identifiers)

    private

    def extract_stack_name(args, options)
      value = args[0] || options.delete(:stack_name)
      case value
      when String then value
      when nil then raise ArgumentError, "missing required option :stack_name"
      else
        msg = "expected :stack_name to be a String, got #{value.class}"
        raise ArgumentError, msg
      end
    end

    def extract_logical_id(args, options)
      value = args[1] || options.delete(:logical_id)
      case value
      when String then value
      when nil then raise ArgumentError, "missing required option :logical_id"
      else
        msg = "expected :logical_id to be a String, got #{value.class}"
        raise ArgumentError, msg
      end
    end

    class Collection < Aws::Resources::Collection; end
  end
end
